#pragma once

#include <iostream>
#include <vector>
#include <boost/functional/hash.hpp>


typedef struct Node 
{
	int nodeID = 0;
	double x = 0, y = 0, z = 0;
	bool vertex = 0;

	Node() = default;
	Node(int ID_, double x_, double y_, double z_, bool vertex_) :
		nodeID(ID_), x(x_), y(y_), z(z_), vertex(vertex_) {}

	friend std::ostream& operator<<(std::ostream&, const Node&);
	friend std::ostream& operator<<(std::ostream&, const std::vector<Node>&);

	bool operator<(const Node& p_node) const 
	{
		return nodeID < p_node.nodeID;
	}

	bool operator==(const Node& p_node) const 
	{
		if (nodeID == p_node.nodeID && x == p_node.x && y == p_node.y &&
			z == p_node.z && vertex == p_node.vertex)
			return true;
		else return false;
	};
}Node;

typedef struct FiniteElement 
{
	int elementID = 0;
	int materialID = 0;
	std::vector<int> nodesID;

	friend std::ostream& operator<<(std::ostream&, const FiniteElement&);
	friend std::ostream& operator<<(std::ostream&, const std::vector<FiniteElement>&);

	bool operator<(const FiniteElement& p_node) const 
	{
		return elementID < p_node.elementID;
	}
}FiniteElement;

typedef struct BoundaryFiniteElement : public FiniteElement 
{
	int boundaryID = 0;
	int surfaceID = 0;

	friend std::ostream& operator<<(std::ostream&, const BoundaryFiniteElement&);
	friend std::ostream& operator<<(std::ostream&, const std::vector<BoundaryFiniteElement>&);
}BoundaryFiniteElement;

typedef struct Edge //    
{ 
	int first_node_id;
	int last_node_id;
	int center_id;

	Edge() = default;
	Edge(int id1, int id2, int id3) :
		first_node_id(id1), last_node_id(id2), center_id(id3) {}

	friend inline bool operator==(const Edge& left_edge, const Edge& right_edge) 
	{
		if (right_edge.first_node_id == left_edge.first_node_id && right_edge.last_node_id == left_edge.last_node_id)
			return true;
		else if (right_edge.first_node_id == left_edge.last_node_id && right_edge.last_node_id == left_edge.first_node_id)
			return true;
		else return false;
	}

	bool operator<(const Edge& right_edge) const 
	{
		return center_id < right_edge.center_id;
	}
	void updateCenter(int id_) 
	{
		center_id = id_;
	}
}Edge;
/*
namespace std 
{
	template<>
	class hash<Edge> 
	{
	public:
		size_t operator()(const Edge& s) const
		{
			std::size_t seed = 0;
			boost::hash_combine(seed, s.first_node_id);
			boost::hash_combine(seed, s.center_id);
			boost::hash_combine(seed, s.last_node_id);
			return seed;
		}
	};
}*/

namespace std
{
	template<>
	class hash<Edge>
	{
	public:
		size_t operator()(const Edge& s) const
		{
			std::size_t seed = 0;
			boost::hash_combine(seed, s.first_node_id);
			boost::hash_combine(seed, s.center_id);
			boost::hash_combine(seed, s.last_node_id);
			return seed;
		}
	};
}